// Copyright (C) 1999 Jean-Marc Valin

#ifndef ROTATING_FRAME_BUFFER_H
#define ROTATING_FRAME_BUFFER_H

#include <vector>
#include "Vector.h"
#include "Object.h"
#include "Buffer.h"
#include "net_types.h"

template <class T>
class RotatingFrameBuffer : public Buffer
{
public:
   RotatingFrameBuffer(int vLength, int bLength=1)
      : Buffer (bLength)
      , vectorLength(vLength)
   {
      for (int i=0;i<bLength;i++) 
      {
	 data[i]= ObjectRef (Vector<T>::alloc(vLength));
         //data[i]= ObjectRef (new Vector<T> (vLength));
         data[i]->status = Object::nil;
      }
   }
   
   inline ObjectRef &operator[] (int ind) 
   {
      if (ind > currentPos)
      {
         for (int i=currentPos+1;i<=ind;i++)
            data[ind % bufferLength]->status = Object::nil;
         currentPos = ind;
      }
      if (ind < 0 || ind <= currentPos-bufferLength)
      {
         //cerr << "ind = " << ind << endl;
         throw new BufferException (this, "trying to access non-existing element",ind);
      }
      return data[ind % bufferLength];
   }
   
   inline ObjectRef operator[] (int ind) const
   {
      if (ind > currentPos)
      {
         for (int i=currentPos+1;i<=ind;i++)
            data[ind % bufferLength]->status = Object::nil;
         currentPos = ind;
      }
      if (ind < 0 || ind <= currentPos-bufferLength)
      {
         throw new BufferException (this, "trying to access non-existing element",ind);
      }
      return data[ind % bufferLength];
   }

protected:
   int vectorLength;
};

#endif
